@@ -1,6 +1,6 @@
 package DBIx::Custom;
 
-our $VERSION = '0.1679';
+our $VERSION = '0.1680';
 
 use 5.008001;
 use strict;
@@ -16,7 +16,7 @@ use DBIx::Custom::QueryBuilder;
 use DBIx::Custom::Where;
 use DBIx::Custom::Model;
 use DBIx::Custom::Tag;
-use DBIx::Custom::Util;
+use DBIx::Custom::Util qw/_array_to_hash _subname/;
 use Encode qw/encode_utf8 decode_utf8/;
 
 use constant DEBUG => $ENV{DBIX_CUSTOM_DEBUG} || 0;
@@ -78,8 +78,8 @@ sub AUTOLOAD {
         $self->dbh->$dbh_method(@_);
     }
     else {
-        croak qq{Can't locate object method "$mname" via "$package"}
-            . qq{ (DBIx::Custom::AUTOLOAD)}
+        croak qq{Can't locate object method "$mname" via "$package" }
+            . _subname;
     }
 }
 
@@ -111,10 +111,10 @@ sub apply_filter {
         
         # Filter infomation
         my $finfo = $cinfos[$i + 1] || {};
-        croak "$usage (table: $table) (DBIx::Custom::apply_filter)"
+        croak "$usage (table: $table) " . _subname
           unless  ref $finfo eq 'HASH';
         foreach my $ftype (keys %$finfo) {
-            croak "$usage (table: $table) (DBIx::Custom::apply_filter)"
+            croak "$usage (table: $table) " . _subname
               unless $ftype eq 'in' || $ftype eq 'out' || $ftype eq 'end'; 
         }
         
@@ -134,8 +134,7 @@ sub apply_filter {
             next if $state eq 'not_exists';
             
             # Check filter name
-            croak qq{Filter "$filter" is not registered}
-                . qq{ (DBIx::Custom::apply_filter)}
+            croak qq{Filter "$filter" is not registered } . _subname
               if  $state eq 'name'
                && ! exists $self->filters->{$filter};
             
@@ -229,7 +228,7 @@ sub create_query {
     
     if ($@) {
         $self->_croak($@, qq{. Following SQL is executed.\n}
-                        . qq{$query->{sql}\n(DBIx::Custom::create_query)});
+                        . qq{$query->{sql}\n} . _subname);
     }
     
     # Set statement handle
@@ -255,8 +254,7 @@ sub dbh {
     else {
         # From Connction manager
         if (my $connector = $self->connector) {
-            croak "connector must have dbh() method"
-                . " (DBIx::Custom::dbh)"
+            croak "connector must have dbh() method " . _subname
               unless ref $connector && $connector->can('dbh');
               
             return $self->{dbh} = $connector->dbh;
@@ -267,33 +265,40 @@ sub dbh {
 }
 
 our %DELETE_ARGS
-  = map { $_ => 1 } @COMMON_ARGS, qw/where append allow_delete_all/;
+  = map { $_ => 1 } @COMMON_ARGS, qw/where append allow_delete_all where_param/;
 
 sub delete {
     my ($self, %args) = @_;
 
     # Check arguments
     foreach my $name (keys %args) {
-        croak qq{"$name" is wrong option}
-            . qq{ (DBIx::Custom::delete)}
+        croak qq{"$name" is wrong option } . _subname
           unless $DELETE_ARGS{$name};
     }
     
     # Arguments
     my $table = $args{table} || '';
-    croak qq{"table" option must be specified. (DBIx::Custom::delete)}
+    croak qq{"table" option must be specified. } . _subname
       unless $table;
     my $where            = delete $args{where} || {};
     my $append           = delete $args{append};
     my $allow_delete_all = delete $args{allow_delete_all};
     my $query_return     = delete $args{query};
+    my $where_param      = delete $args{where_param} || {};
 
     # Where
-    $where = $self->_where_to_obj($where);
-    
-    # Where clause
-    my $where_clause = $where->to_string;
-    croak qq{"where" must be specified (DBIx::Custom::delete)}
+    my $where_clause = '';
+    if (ref $where) {
+        $where = $self->_where_to_obj($where);
+        $where_param = keys %$where_param
+                     ? $self->merge_param($where_param, $where->param)
+                     : $where->param;
+        
+        # String where
+        $where_clause = $where->to_string;
+    }
+    elsif ($where) { $where_clause = "where $where" }
+    croak qq{"where" must be specified } . _subname
       if $where_clause eq '' && !$allow_delete_all;
 
     # Delete statement
@@ -310,7 +315,7 @@ sub delete {
     # Execute query
     return $self->execute(
         $query,
-        param => $where->param,
+        param => $where_param,
         table => $table,
         %args
     );
@@ -330,7 +335,7 @@ sub delete_at {
     
     # Check arguments
     foreach my $name (keys %args) {
-        croak qq{"$name" is wrong option (DBIx::Custom::delete_at)}
+        croak qq{"$name" is wrong option } . _subname
           unless $DELETE_AT_ARGS{$name};
     }
     
@@ -365,8 +370,7 @@ sub create_model {
     $self->apply_filter($model->table, @$filter);
     
     # Associate table with model
-    croak "Table name is duplicated"
-        . " (DBIx::Custom::create_model)"
+    croak "Table name is duplicated " . _subname
       if exists $self->{_model_from}->{$model->table};
     $self->{_model_from}->{$model->table} = $model->name;
 
@@ -409,13 +413,13 @@ sub execute {
     my $tables = delete $args{table} || [];
     $tables = [$tables] unless ref $tables eq 'ARRAY';
     my $filter = delete $args{filter};
-    $filter = DBIx::Custom::Util::array_to_hash($filter);
+    $filter = _array_to_hash($filter);
     my $type = delete $args{type};
-    $type = DBIx::Custom::Util::array_to_hash($type);
+    $type = _array_to_hash($type);
     
     # Check argument names
     foreach my $name (keys %args) {
-        croak qq{"$name" is wrong option (DBIx::Custom::execute)}
+        croak qq{"$name" is wrong option } . _subname
           unless $EXECUTE_ARGS{$name};
     }
     
@@ -474,8 +478,7 @@ sub execute {
             $filter->{$column} = undef;
         }
         elsif (ref $name ne 'CODE') {
-          croak qq{Filter "$name" is not registered"}
-              . qq{ (DBIx::Custom::execute}
+          croak qq{Filter "$name" is not registered" } . _subname
             unless exists $self->filters->{$name};
           $filter->{$column} = $self->filters->{$name};
         }
@@ -502,7 +505,7 @@ sub execute {
     
     if ($@) {
         $self->_croak($@, qq{. Following SQL is executed.\n}
-                        . qq{$query->{sql}\n});
+                        . qq{$query->{sql}\n} . _subname);
     }
     
     # Output SQL for debug
@@ -548,7 +551,7 @@ sub insert {
     
     # Arguments
     my $table  = delete $args{table};
-    croak qq{"table" option must be specified (DBIx::Custom::insert)}
+    croak qq{"table" option must be specified } . _subname
       unless $table;
     my $param  = delete $args{param} || {};
     my $append = delete $args{append} || '';
@@ -556,7 +559,7 @@ sub insert {
 
     # Check arguments
     foreach my $name (keys %args) {
-        croak qq{"$name" is wrong option (DBIx::Custom::insert)}
+        croak qq{"$name" is wrong option } . _subname
           unless $INSERT_ARGS{$name};
     }
 
@@ -567,8 +570,7 @@ sub insert {
     my @columns;
     my $safety = $self->safety_character;
     foreach my $column (keys %$param) {
-        croak qq{"$column" is not safety column name}
-            . qq{ (DBIx::Custom::insert)}
+        croak qq{"$column" is not safety column name } . _subname
           unless $column =~ /^[$safety\.]+$/;
           $column = "$q$column$q";
           $column =~ s/\./$q.$q/;
@@ -607,7 +609,7 @@ sub insert_at {
     
     # Check arguments
     foreach my $name (keys %args) {
-        croak qq{"$name" is wrong option (DBIx::Custom::insert_at)}
+        croak qq{"$name" is wrong option } . _subname
           unless $INSERT_AT_ARGS{$name};
     }
     
@@ -627,8 +629,7 @@ sub insert_param_tag {
     my @columns;
     my @placeholders;
     foreach my $column (keys %$param) {
-        croak qq{"$column" is not safety column name}
-            . qq{ (DBIx::Custom::insert_param_tag) }
+        croak qq{"$column" is not safety column name } . _subname
           unless $column =~ /^[$safety\.]+$/;
         $column = "$q$column$q";
         $column =~ s/\./$q.$q/;
@@ -650,20 +651,18 @@ sub include_model {
     unless ($model_infos) {
 
         # Load name space module
-        croak qq{"$name_space" is invalid class name}
-            . qq{ (DBIx::Custom::include_model)}
+        croak qq{"$name_space" is invalid class name } . _subname
           if $name_space =~ /[^\w:]/;
         eval "use $name_space";
-        croak qq{Name space module "$name_space.pm" is needed. $@}
-            . qq{ (DBIx::Custom::include_model)}
+        croak qq{Name space module "$name_space.pm" is needed. $@ }
+            . _subname
           if $@;
         
         # Search model modules
         my $path = $INC{"$name_space.pm"};
         $path =~ s/\.pm$//;
         opendir my $dh, $path
-          or croak qq{Can't open directory "$path": $!}
-                 . qq{ (DBIx::Custom::include_model)};
+          or croak qq{Can't open directory "$path": $! } . _subname
         $model_infos = [];
         while (my $module = readdir $dh) {
             push @$model_infos, $module
@@ -689,12 +688,11 @@ sub include_model {
         }
         else { $model_class = $model_name = $model_table = $model_info }
         my $mclass = "${name_space}::$model_class";
-        croak qq{"$mclass" is invalid class name}
-            . qq{ (DBIx::Custom::inculde_model)}
+        croak qq{"$mclass" is invalid class name } . _subname
           if $mclass =~ /[^\w:]/;
         unless ($mclass->can('isa')) {
             eval "use $mclass";
-            croak "$@ (DBIx::Custom::include_model)" if $@;
+            croak "$@ " . _subname if $@;
         }
         
         # Create model
@@ -749,7 +747,7 @@ sub model {
     }
     
     # Check model existance
-    croak qq{Model "$name" is not included (DBIx::Custom::model)}
+    croak qq{Model "$name" is not included } . _subname
       unless $self->models->{$name};
     
     # Get model
@@ -774,7 +772,7 @@ sub new {
     # Check attributes
     my @attrs = keys %$self;
     foreach my $attr (@attrs) {
-        croak qq{"$attr" is wrong name (DBIx::Custom::new)}
+        croak qq{"$attr" is wrong name } . _subname
           unless $self->can($attr);
     }
     
@@ -812,7 +810,7 @@ sub register_tag { shift->query_builder->register_tag(@_) }
 
 our %SELECT_ARGS
   = map { $_ => 1 } @COMMON_ARGS,
-                    qw/column where append relation join param wrap/;
+                    qw/column where append relation join param where_param wrap/;
 
 sub select {
     my ($self, %args) = @_;
@@ -826,17 +824,19 @@ sub select {
     my $where     = delete $args{where} || {};
     my $append    = delete $args{append};
     my $join      = delete $args{join} || [];
-    croak qq{"join" must be array reference}
-        . qq{ (DBIx::Custom::select)}
+    croak qq{"join" must be array reference } . _subname
       unless ref $join eq 'ARRAY';
     my $relation = delete $args{relation};
-    my $param = delete $args{param} || {};
+    my $param = delete $args{param} || {}; # DEPRECATED!
+    warn "DEPRECATED select() param option. this is renamed to where_param"
+      if keys %$param;
+    my $where_param = delete $args{where_param} || $param || {};
     my $query_return = $args{query};
     my $wrap = delete $args{wrap};
 
     # Check arguments
     foreach my $name (keys %args) {
-        croak qq{"$name" is wrong option (DBIx::Custom::select)}
+        croak qq{"$name" is wrong option } . _subname
           unless $SELECT_ARGS{$name};
     }
     
@@ -873,19 +873,25 @@ sub select {
         push @sql, "$q$main_table$q";
     }
     pop @sql if ($sql[-1] || '') eq ',';
-    croak "Not found table name (DBIx::Custom::select)"
+    croak "Not found table name " . _subname
       unless $tables->[-1];
 
     # Add tables in parameter
-    unshift @$tables, @{$self->_search_tables(join(' ', keys %$param) || '')};
+    unshift @$tables,
+            @{$self->_search_tables(join(' ', keys %$where_param) || '')};
     
     # Where
-    $where = $self->_where_to_obj($where);
-    $param = keys %$param ? $self->merge_param($param, $where->param)
-                          : $where->param;
-    
-    # String where
-    my $where_clause = $where->to_string;
+    my $where_clause = '';
+    if (ref $where) {
+        $where = $self->_where_to_obj($where);
+        $where_param = keys %$where_param
+                     ? $self->merge_param($where_param, $where->param)
+                     : $where->param;
+        
+        # String where
+        $where_clause = $where->to_string;
+    }
+    elsif ($where) { $where_clause = "where $where" }
     
     # Add table names in where clause
     unshift @$tables, @{$self->_search_tables($where_clause)};
@@ -904,7 +910,7 @@ sub select {
     
     # Wrap
     if ($wrap) {
-        croak "wrap option must be array refrence (DBIx::Custom::select)"
+        croak "wrap option must be array refrence " . _subname
           unless ref $wrap eq 'ARRAY';
         unshift @sql, $wrap->[0];
         push @sql, $wrap->[1];
@@ -920,7 +926,7 @@ sub select {
     # Execute query
     my $result = $self->execute(
         $query,
-        param => $param, 
+        param => $where_param, 
         table => $tables,
         %args
     );
@@ -941,12 +947,12 @@ sub select_at {
     
     # Check arguments
     foreach my $name (keys %args) {
-        croak qq{"$name" is wrong option (DBIx::Custom::select_at)}
+        croak qq{"$name" is wrong option } . _subname
           unless $SELECT_AT_ARGS{$name};
     }
     
     # Table
-    croak qq{"table" option must be specified (DBIx::Custom::select_at)}
+    croak qq{"table" option must be specified } . _subname
       unless $args{table};
     my $table = ref $args{table} ? $args{table}->[-1] : $args{table};
     
@@ -972,23 +978,24 @@ sub setup_model {
 }
 
 our %UPDATE_ARGS
-  = map { $_ => 1 } @COMMON_ARGS, qw/param where append allow_update_all/;
+  = map { $_ => 1 } @COMMON_ARGS, qw/param where append allow_update_all where_param/;
 
 sub update {
     my ($self, %args) = @_;
 
     # Arguments
     my $table = delete $args{table} || '';
-    croak qq{"table" option must be specified (DBIx::Custom::update)}
+    croak qq{"table" option must be specified } . _subname
       unless $table;
     my $param            = delete $args{param} || {};
     my $where            = delete $args{where} || {};
+    my $where_param      = delete $args{where_param} || {};
     my $append           = delete $args{append} || '';
     my $allow_update_all = delete $args{allow_update_all};
     
     # Check argument names
     foreach my $name (keys %args) {
-        croak qq{"$name" is wrong option (DBIx::Custom::update)}
+        croak qq{"$name" is wrong option } . _subname
           unless $UPDATE_ARGS{$name};
     }
     
@@ -997,8 +1004,7 @@ sub update {
     my $safety = $self->safety_character;
     my $q = $self->reserved_word_quote;
     foreach my $column (keys %$param) {
-        croak qq{"$column" is not safety column name}
-            . qq{ (DBIx::Custom::update)}
+        croak qq{"$column" is not safety column name } . _subname
           unless $column =~ /^[$safety\.]+$/;
           $column = "$q$column$q";
           $column =~ s/\./$q.$q/;
@@ -1009,19 +1015,28 @@ sub update {
     my $update_clause = '{update_param ' . join(' ', @columns) . '}';
 
     # Where
-    $where = $self->_where_to_obj($where);
-    my $where_clause = $where->to_string;
-    croak qq{"where" must be specified (DBIx::Custom::update)}
+    my $where_clause = '';
+    if (ref $where) {
+        $where = $self->_where_to_obj($where);
+        $where_param = keys %$where_param
+                     ? $self->merge_param($where_param, $where->param)
+                     : $where->param;
+        
+        # String where
+        $where_clause = $where->to_string;
+    }
+    elsif ($where) { $where_clause = "where $where" }
+    croak qq{"where" must be specified } . _subname
       if "$where_clause" eq '' && !$allow_update_all;
     
+    # Merge param
+    $param = $self->merge_param($param, $where_param) if keys %$where_param;
+    
     # Update statement
     my @sql;
     push @sql, "update $q$table$q $update_clause $where_clause";
     push @sql, $append if $append;
     
-    # Merge parameters
-    $param = $self->merge_param($param, $where->param);
-    
     # SQL
     my $sql = join(' ', @sql);
     
@@ -1055,7 +1070,7 @@ sub update_at {
 
     # Check arguments
     foreach my $name (keys %args) {
-        croak qq{"$name" is wrong option (DBIx::Custom::update_at)}
+        croak qq{"$name" is wrong option } . _subname
           unless $UPDATE_AT_ARGS{$name};
     }
     
@@ -1073,8 +1088,7 @@ sub update_param_tag {
     my $safety = $self->safety_character;
     my $q = $self->reserved_word_quote;
     foreach my $column (keys %$param) {
-        croak qq{"$column" is not safety column name}
-            . qq{ (DBIx::Custom::update_param_tag) }
+        croak qq{"$column" is not safety column name } . _subname
           unless $column =~ /^[$safety\.]+$/;
         my $column = "$q$column$q";
         $column =~ s/\./$q.$q/;
@@ -1172,7 +1186,7 @@ sub _connect {
     
     # Attributes
     my $data_source = $self->data_source;
-    croak qq{"data_source" must be specified (DBIx::Custom::dbh)"}
+    croak qq{"data_source" must be specified } . _subname
       unless $data_source;
     my $user        = $self->user;
     my $password    = $self->password;
@@ -1190,7 +1204,7 @@ sub _connect {
     )};
     
     # Connect error
-    croak "$@ (DBIx::Custom::dbh)" if $@;
+    croak "$@ " . _subname if $@;
     
     return $dbh;
 }
@@ -1246,15 +1260,14 @@ sub _push_join {
         if ($join_clause =~ $join_re) {
             my $table1 = $1;
             my $table2 = $2;
-            croak qq{right side table of "$join_clause" must be unique}
-                . qq{ (DBIx::Custom::select)}
+            croak qq{right side table of "$join_clause" must be unique }
+                . _subname
               if exists $tree->{$table2};
             $tree->{$table2}
               = {position => $i, parent => $table1, join => $join_clause};
         }
         else {
-            croak qq{join "$join_clause" must be two table name}
-                . qq{ (DBIx::Custom::select)};
+            croak qq{join "$join_clause" must be two table name } . _subname
         }
     }
     
@@ -1332,7 +1345,7 @@ sub _where_to_obj {
     # Check where argument
     croak qq{"where" must be hash reference or DBIx::Custom::Where object}
         . qq{or array reference, which contains where clause and paramter}
-        . " (" . (caller(1))[3] . ")"
+        . _subname
       unless ref $obj eq 'DBIx::Custom::Where';
     
     return $obj;
@@ -1899,12 +1912,10 @@ or array refrence, which contains where clause and paramter.
     );
     $dbi->delete(where => $where);
 
-    # Array refrendce (where clause and parameter)
-    $dbi->delete(where =>
-        [
-            ['and', '{= author}', '{like title}'],
-            {author => 'Ken', title => '%Perl%'}
-        ]
+    # String(with where_param option)
+    $dbi->delete(
+        where => '{like title}',
+        where_param => {title => '%Perl%'}
     );
     
 =item C<append>
@@ -2370,12 +2381,10 @@ or array refrence, which contains where clause and paramter.
     );
     $dbi->select(where => $where);
 
-    # Array refrendce (where clause and parameter)
-    $dbi->select(where =>
-        [
-            ['and', '{= author}', '{like title}'],
-            {author => 'Ken', title => '%Perl%'}
-        ]
+    # String(with where_param option)
+    $dbi->select(
+        where => '{like title}',
+        where_param => {title => '%Perl%'}
     );
     
 =item C<join>
@@ -2582,14 +2591,13 @@ or array refrence.
     );
     $dbi->update(where => $where);
     
-    # Array refrendce (where clause and parameter)
-    $dbi->update(where =>
-        [
-            ['and', '{= author}', '{like title}'],
-            {author => 'Ken', title => '%Perl%'}
-        ]
+    # String(with where_param option)
+    $dbi->update(
+        param => {title => 'Perl'},
+        where => '{= id}',
+        where_param => {id => 2}
     );
-
+    
 =item C<append>
 
 Append statement to last of SQL. This is string.
